/*
 * display.c -  .
 */
#include <asm/io.h>
#include <asm/system.h>
#include <display.h>

/* VGA index register port */
#define VGA_CRT_AR          0x3d4    /* CRT Controller Address Register */
#define VGA_CRT_DR          0x3d5    /* CRT Controller Data Register */

/* VGA CRT controller register */
#define VGA_CRTC_START_HI    0x0c
#define VGA_CRTC_START_LO    0x0d
#define VGA_CRTC_CURSOR_HI   0x0e
#define VGA_CRTC_CURSOR_LO   0x0f

#define SCREN_START      0x0b8000
#define SCREN_END        0x0c0000

#define LINES                  25
#define COLUMNS                80

#define ATTR                 0x07

/*    */
static void set_cursor(void);
static void set_scr_start(void);
static void scrup(void);

/*   */
static unsigned int scr_start = SCREN_START;
static unsigned int scr_end = SCREN_START+COLUMNS*LINES*2;
static unsigned char lines, columns;

/*    */
void set_cursor(void)
{
	unsigned int pos = ((scr_start - SCREN_START) >> 1) +
		(lines * COLUMNS) + columns;

	outb(VGA_CRTC_CURSOR_HI,VGA_CRT_AR);
	outb((pos >> 8) & 0xff,VGA_CRT_DR);
	outb(VGA_CRTC_CURSOR_LO,VGA_CRT_AR);
	outb(pos & 0xff,VGA_CRT_DR);
}

/*     */
void set_scr_start(void)
{
	unsigned int scr = (scr_start - SCREN_START) >> 1;

	outb(VGA_CRTC_START_HI,VGA_CRT_AR);
	outb((scr >> 8) & 0xff,VGA_CRT_DR);
	outb(VGA_CRTC_START_LO,VGA_CRT_AR);
	outb(scr & 0xff,VGA_CRT_DR);
}

/*    */
void scrup(void)
{
	int i;

	/*    */
	if( (++columns >= COLUMNS) && (++lines >= LINES) )
	{
		/*      */
		scr_start += COLUMNS * 2;
		scr_end += COLUMNS * 2;
		/*       */
		columns = 0;
		lines = LINES - 1;
		/*      */
		if( scr_end > SCREN_END )
		{
			/*     */
			for( i=0; i < (COLUMNS * (LINES - 1)); i++ )
				*((unsigned short *)((i*2)+SCREN_START)) = 
					*((unsigned short *)((i*2)+scr_start));

			/*     */
			scr_start = SCREN_START;
			scr_end = SCREN_START + COLUMNS * LINES * 2;
		}

		/*    */
		for( i=0; i < COLUMNS; i++ )
			*((unsigned short *)(scr_start+((lines*COLUMNS+i)*2))) = 0x0720;

		/*     */
		set_scr_start();
	}
}

/*   */
void display_init(void)
{
	/*      */
	columns = *((unsigned char *)0x0450);
	lines = *((unsigned char *)0x0451);

	puts("Display init OK...\n");
}

/*      */
int putchar(int ch)
{
	unsigned char tmp = ch & 0xff;
	unsigned int flags;

	/*    */
	flags = get_flags();
	cli();
	{
		/*     */
		if( tmp == '\n' )
			columns = COLUMNS - 1;
		else
			*((unsigned short int *)(scr_start + (((lines * COLUMNS) +
							columns) << 1))) = ch | 0x0700;

		/*    */
		scrup();
		/*   */
		set_cursor();
	}
	set_flags(flags);

	return ch;
}

/*   */
int puts(const char *str)
{
	int i;

	for( i=0; str[i]; i++ )
		putchar(str[i]);

	return i;
}
