/*
 * irq.c -  
 */
#include <asm/io.h>
#include <kernel.h>
#include <irq.h>
#include <display.h>

extern void irq_reserved(void);

/*    */
void set_irq(int i, void (* func)(void))
{
	if( i >= 16 )
		return;

	idt[i+32].a = (8 << 16) | ((unsigned int)func & 0xffff);
	idt[i+32].b = ((unsigned int)func & 0xffff0000) | 0x8e00;

	if( i < 8 )
		outb(inb(0x21)&~(1<<i),0x21);
	else
		outb(inb(0xa1)&~(1<<(i-8)),0xa1);
}

void do_irq_reserved(void)
{
	puts("IRQ not used...\n");
	for(;;);
}

/*   */
void irq_init(void)
{
	int i;

	/*     */
	outb(0x11,0x20);   /*  1 PIC */
	outb(0x11,0xa0);   /*  2 PIC */
	outb(0x20,0x21);   /*    0x20 PIC 1 */
	outb(0x28,0xa1);   /*    0x28 PIC 2 */
	outb(0x04,0x21);   /* PIC 1 master */
	outb(0x02,0xa1);   /* PIC 2 slave */
	outb(0x01,0x21);   /*  8086  PIC 1 */
	outb(0x01,0xa1);   /*  8086  PIC 2 */
	outb(0xff,0x21);   /*     PIC 1 */
	outb(0xff,0xa1);   /*     PIC 2 */

	for( i=32; i < 48; i++ )
		set_irq(i,irq_reserved);

	puts("IRQ init OK...\n");
}
