/*
 * main.c
 */
#include <asm/io.h>
#include <asm/system.h>
#include <display.h>
#include <irq.h>
#include <traps.h>
#include <keyboard.h>
#include <timer.h>
#include <syscall.h>
#include <task.h>
#include <kernel.h>

void func_1(void);
void func_2(void);

volatile int f1,f2;

static inline int get_cs(void)
{
	int v = 0;
	__asm__ __volatile__ (
			"xorl %%eax, %%eax\n"
			"movw %%cs, %%ax\n"
			: "=a" (v) :);
	return v;
}

static inline int get_ss(void)
{
	int v = 0;
	__asm__ __volatile__ (
			"xorl %%eax, %%eax\n"
			"movw %%ss, %%ax\n"
			: "=a" (v) :);
	return v;
}

static inline int get_ds(void)
{
	int v = 0;
	__asm__ __volatile__ (
			"xorl %%eax, %%eax\n"
			"movw %%ds, %%ax\n"
			: "=a" (v) :);
	return v;
}

static int local_putchar(int a)
{
	return syscall(2,a);
}

int main(void)
{
	display_init();
	irq_init();
	traps_init();
	timer_init();
	syscall_init();
	keyboard_init();
	task_set(func_1);
	task_set(func_2);
	sti();

	/*
	 *     
	 *    .
	 */
	for(;;);

	return 0;
}

/*
 *  ,     
 *     . (. task.c)
 */
void func_1(void)
{
	int i,j;
//	static unsigned int tick;

	for(;;)
	{
		for( i=0; i < 17777; i++ )
			for( j=0; j < 50000; j++ )
				if( f1 )
				{
					unsigned int flags = get_flags();
					cli();
					{
						printk("task 1: cs = %x; ss = %x; ds = %x;\n",
								get_cs(),get_ss(),get_ds());
						f1 = 0;
					}
					set_flags(flags);
				}
		local_putchar('A');
//		printk("func 1. tick = %u\n",tick++);
	}
}

/*  . */
void func_2(void)
{
	int i,j;
//	static unsigned int tick;

	for(;;)
	{
		for( i=0; i < 10000; i++ )
			for( j=0; j < 50000; j++ )
				if( f2 )
				{
					unsigned int flags = get_flags();
					cli();
					{
						printk("task 2: cs = %x; ss = %x; ds = %x;\n",
								get_cs(),get_ss(),get_ds());
						f2 = 0;
					}
					set_flags(flags);
				}
		local_putchar('B');
//		printk("func 2. tick = %u\n",tick++);
	}
}
