#include <kernel.h>
#include <display.h>
#include <stdarg.h>

#define L         0
#define H         1

/*   */
static int print_d(long int val);
static int print_u(unsigned long int val);
static int print_x(unsigned long int val, int caps);
static int print_p(void *point);

/*
 *  "printk"   "printf"
 *     "C".
 *    :
 * %c %d %i %s %u %x %X %p %n %% .
 *    :
 * l h .
 */
int printk(const char *format, ...)
{
	va_list args;
	int count = 0;

	va_start(args,format);
	for( ; *format; format++ )
	{
		if( *format == '%' )
		{
			format++;
			switch( *format )
			{
				case 'c': putchar(va_arg(args,char)); count++; break;
				case 'd':
				case 'i': count += print_d(va_arg(args,int)); break;
				case 's': count += puts(va_arg(args,char *)); break;
				case 'u': count += print_u(va_arg(args,unsigned int)); break;
				case 'x': count += print_x(va_arg(args,unsigned int),L); break;
				case 'X': count += print_x(va_arg(args,unsigned int),H); break;
				case 'p': count += print_p(va_arg(args,void *)); break;
				case 'n': *((int *)va_arg(args,int *)) = count; break;
				case '%': putchar('%'); count++; break;

				case 'l':
					format++;
					switch( *format )
					{
						case 'd':
						case 'i': count += print_d(va_arg(args,long int)); break;
						case 'u': count += print_u(va_arg(args,unsigned long int)); break;
						case 'x': count += print_x(va_arg(args,unsigned long int),L); break;
						case 'X': count += print_x(va_arg(args,unsigned long int),H); break;
					}
					break;

				case 'h':
					format++;
					switch( *format )
					{
						case 'd':
						case 'i': count += print_d(va_arg(args,short int)); break;
						case 'u': count += print_u(va_arg(args,unsigned short int)); break;
						case 'x': count += print_x(va_arg(args,unsigned short int),L); break;
						case 'X': count += print_x(va_arg(args,unsigned short int),H); break;
					}
					break;
			}
		}
		else
		{
			putchar(*format);
			count++;
		}
	}
	va_end(args);

	return count;
}

/*        */
int print_d(long int val)
{
	int count = 0;

	if( val < 0 )
	{
		putchar('-');
		val = -val;
		count++;
	}

	return count + print_u(val);
}

/*      */
int print_u(unsigned long int val)
{
	char buff[10];
	int count, i = 0;

	do
	{
		buff[i++] = val % 10 + '0';
		val /= 10;
	} while( val );

	for( count=0; i; i--, count++ )
		putchar(buff[i-1]);

	return count;
}

/*     */
int print_x(unsigned long int val, int caps)
{
	char buff[8];
	int count, tmp, i = 0;

	do
	{
		tmp = val % 16;
		buff[i++] = tmp < 0x0a ? tmp + '0' : caps ? tmp - 0x0a + 'A' : tmp - 0x0a + 'a';
		val /= 16;
	} while( val );

	for( count=0; i; i--, count++ )
		putchar(buff[i-1]);

	return count;
}

/*   */
int print_p(void *point)
{
	char buff[8];
	int count, tmp, i = 0;
	unsigned long int val = (unsigned long int)point;

	do
	{
		tmp = val % 16;
		buff[i++] = tmp < 0x0a ? tmp + '0' : tmp - 0x0a + 'a';
		val /= 16;
	} while( val );

	for( ; i < 8; i++ )
		buff[i] = '0';

	count = puts("0x");
	for( ; i; i--, count++ )
		putchar(buff[i-1]);

	return count;
}
