#include <kernel.h>
#include <display.h>

struct tss_struct
{
	unsigned int link;
	unsigned int esp0, ss0;
	unsigned int esp1, ss1;
	unsigned int esp2, ss2;
	unsigned int cr3;
	unsigned int eip;
	unsigned int eflags;
	unsigned int eax, ecx, edx, ebx;
	unsigned int esp, ebp, esi, edi;
	unsigned int es, cs, ss, ds, fs, gs;
	unsigned int ldtr;
	unsigned int map;
};

struct ldtr_d
{
	struct desc null;
	struct desc code;
	struct desc data;
};

extern unsigned int *page_dir;

static struct tss_struct tss[3];
static struct ldtr_d ldtr[2];
static char use[3];

void task_set(void (* func)(void))
{
	int i;

	for( i=0; i < 2; i++ )
	{
		if( !use[i] )
		{
			gdt[i*2+3].a = (sizeof(struct tss_struct) - 1) | ((unsigned int)&tss[i] << 16);
			gdt[i*2+3].b = (((unsigned int)&tss[i] >> 16) & 0xff) | (0x89 << 8) |
				((unsigned int)&tss[i] & (0xff << 24));

			gdt[i*2+4].a = (sizeof(struct ldtr_d) - 1) | ((unsigned int)&ldtr[i] << 16);
			gdt[i*2+4].b = (((unsigned int)&ldtr[i] >> 16) & 0xff) | (0x82 << 8) |
				((unsigned int)&ldtr[i] & (0xff << 24));

			ldtr[i].null.a = 0;
			ldtr[i].null.b = 0;

			ldtr[i].code.a = 0x000000bf;
			ldtr[i].code.b = 0x00c0fa00;

			ldtr[i].data.a = 0x000000bf;
			ldtr[i].data.b = 0x00c0f200;

			tss[i].esp0 = 0x80000 - 0x10000 * i;
			tss[i].ss0 = 16;

			tss[i].cr3 = (unsigned int)&page_dir;
			tss[i].eip = (unsigned int)func;
			tss[i].eflags = (1 << 9);
			tss[i].esp = 0xa0000 - 0x10000 * i;
			tss[i].cs = 0x0b | 4;
			tss[i].ss = 0x13 | 4;
			tss[i].ds = 0x13 | 4;
			tss[i].ldtr = (i*2+4)*8;

			use[i] = 1;

			break;
		}
	}
}

void task_show(struct tss_struct *p)
{
	printk("link = %p;\n"
			"esp0 = %p; ss0 = %p;\n"
			"esp1 = %p; ss1 = %p;\n"
			"esp2 = %p; ss2 = %p;\n"
			"cr3 = %p; eip = %p; eflags = %p;\n"
			"eax = %p; ecx = %p; edx = %p; ebx = %p;\n"
			"esp = %p; ebp = %p; esi = %p; edi = %p;\n"
			"es = %x; cs = %x; ss = %x; ds = %x; fs = %x; gs = %x;\n"
			"ldtr = %p; map = %p;\n",
			p->link,
			p->esp0,p->ss0,
			p->esp1,p->ss1,
			p->esp2,p->ss2,
			p->cr3,p->eip,p->eflags,
			p->eax,p->ecx,p->edx,p->ebx,
			p->esp,p->ebp,p->esi,p->edi,
			p->es,p->cs,p->ss,p->ds,p->fs,p->gs,
			p->ldtr,p->map);
}

void task_print(void)
{
	task_show(&tss[0]);
	putchar('\n');
	task_show(&tss[1]);
}

void task_switch(unsigned int *adr)
{
	static volatile int i;
	static volatile int task;

	if( i++ < 2 )
	{
		if( task != 1 )
		{
			task = 1;
			__asm__ __volatile__ ("ljmp $24, $0" ::);
		}
	}
	else
	{
		i = 0; task = 0;
		__asm__ __volatile__ ("ljmp $40, $0\n" ::);
	}
}
